function  [leak_start, leak_end, write_start, write_end, read_start, read_end] = writeStimFile_DFFRAM_64B( filename )
%generates a STIM file for use in Kai Likne's testbenches of a D-flip-flop
%64 byte RAM circuit
%% constants and initialization:
hier = '0';
tunit = 'ns';
trise = '0.001' ; %100ps falltime/risetime
tfall = '0.001' ;
vih = '3';
vil = '0';
period = 100; %10Mhz clock freq, 100ns period
wait_time = 1000000000; %1 second wait time to measure leak

radix_data = '44';
radix_address = '24';
radix_store = '1';
radix_cdn = '1';

time = '000000000000';
time_amount_of_digits = length(time); %has to be 12
maxtime = 10^(time_amount_of_digits);
randomdata='00';
address_string='00';

%% start writing file
fileID = fopen(filename,'w');

fprintf(fileID,'radix\n');
fprintf(fileID,'+ %s %s %s %s\n', radix_data, radix_address, radix_store, radix_cdn);
fprintf(fileID,'\nio\n');
fprintf(fileID,'+ i i i i\n\n');
fprintf(fileID,'hier %s\ntunit %s\ntrise %s\ntfall %s\nvih %s\nvil %s\n\n', hier, tunit, trise, tfall, vih, vil);
fprintf(fileID,'\nvname\n+DATA_BUS_VECTOR<[7:0]> ADDRESS_VECTOR<[5:0]> STORE_VECTOR CDN_VECTOR\n\n');


fprintf(fileID,'\n\n\n; Initializing\n\n');
fprintf(fileID,';time\t\tdata\tad\tst\tcdn\n');
fprintf(fileID,'%s\t%s\t%s\t0\t0\t; Reset state\n',time,randomdata,address_string); %enter reset state
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t0\t1\t; deactivate reset state\n',time,randomdata,address_string);
time = sprintf('%012d',str2num(time)+period*10); %timestep 2 clock periods
  
%% measure leakage current
time = sprintf('%012d',str2num(time)+wait_time); %timestep 1 second to measure leak
leak_start = time;
time = sprintf('%012d',str2num(time)+ceil(wait_time/4)); %timestep 0.25 seconds
leak_end = time;
time = sprintf('%012d',str2num(time)+period); %timestep one clock period


%% measure write energy

fprintf(fileID,'\n\n\n; Start measuring write energy\n\n');
fprintf(fileID,';time\t\tdata\tad\tst\tcdn\n');
time = sprintf('%012d',str2num(time)+ceil(wait_time/4)); %timestep 0.25 seconds

write_start = time;

address = '0A'   ;
data = 'FF';
fprintf(fileID,'%s\t%s\t%s\t0\t1\t; Setup time\n',time,data,address);
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t1\t1\t; Write %s to Address %s\n',time,data,address,data,address);
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t0\t1\t; hold time\n',time,data,address);
time = sprintf('%012d',str2num(time)+period); %timestep one clock period

fprintf(fileID,'%s\t00\t00\t0\t1\t; address and data are 0 when idle\n',time);
time = sprintf('%012d',str2num(time)+period); %timestep one clock period

write_end = time;

%% measure read energy
fprintf(fileID,'\n\n\n; Start measuring read energy\n\n');
fprintf(fileID,';time\t\tdata\tad\tst\tcdn\n');
time = sprintf('%012d',str2num(time)+ceil(wait_time/4)); %timestep 0.25 seconds


address = '0A' ; 
data = '00'; %N/A

read_start = time;

fprintf(fileID,'%s\t%s\t%s\t0\t1\t; measure read energy. start read at address %s\n',time,data,address,address);
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t00\t00\t0\t1\t; read end address and data are 0 when idle\n',time);
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
read_end = time;

%% print out stats and close file
fprintf(fileID,'\n\n\n; leak_start = %s \t leak_end = %s \n; write_start = %s \t write_end = %s\n; read_start = %s \t read_end = %s\n',leak_start, leak_end, write_start,write_end,read_start,read_end);
fclose(fileID);
end

