function  [leak_start, leak_end, write_start, write_end, read_start, read_end] = writeStimFile_SRAM6T_64B( filename )
%generates a STIM file for use in Kai Likne's testbenches of a 6T SRAM
%64 byte circuit
%% constants and initialization:
hier = '0';
tunit = 'ns';
trise = '0.001' ; %100ps falltime/risetime
tfall = '0.001' ;
vih = '3';
vil = '0';
period = 100; %10Mhz clock freq, 100ns period
wait_time = 1000000000; %1 second wait time to measure leak

radix_data = '44';
radix_row_address = '4';
radix_col_address = '2';
radix_read = '1';
radix_write = '1';
radix_WL_ENABLE = '1';
radix_cdn = '1';

time = '000000000000';
time_amount_of_digits = length(time); %has to be 12
maxtime = 10^(time_amount_of_digits);
data='00';
row_address='0';
col_address='0';

%% start writing file
fileID = fopen(filename,'w');

fprintf(fileID,'radix\n');
fprintf(fileID,'+ %s %s %s %s %s %s %s\n', radix_data, radix_row_address, radix_col_address, radix_read, radix_write, radix_WL_ENABLE, radix_cdn);
fprintf(fileID,'\nio\n');
fprintf(fileID,'+ i i i i i i i\n\n');
fprintf(fileID,'hier %s\ntunit %s\ntrise %s\ntfall %s\nvih %s\nvil %s\n\n', hier, tunit, trise, tfall, vih, vil);
fprintf(fileID,'\nvname\n+DATA_BUS_VECTOR<[7:0]> ROW_ADDRESS_VECTOR<[3:0]> COL_ADDRESS_VECTOR<[1:0]> READ_VECTOR WRITE_VECTOR WL_ENABLE_VECTOR CDN_VECTOR\n\n');


fprintf(fileID,'\n\n\n; Initializing\n\n');
fprintf(fileID,';time\t\tdata\trow\tcol\trd\twrt\twl\tcdn\n');
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; Reset state\n',time,data,row_address,col_address,'0','0','0','0'); %enter reset state
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; exit out of reset state\n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period*10); %timestep 2 clock periods
  
%% measure leakage current
time = sprintf('%012d',str2num(time)+wait_time); %timestep 1 second to measure leak
 leak_start = time;
 time = sprintf('%012d',str2num(time)+ceil(wait_time/4)); %timestep 0.25 seconds
 leak_end = time;
time = sprintf('%012d',str2num(time)+period); %timestep one clock period


%% measure write energy

fprintf(fileID,'\n\n\n; Start measuring write energy\n\n');
fprintf(fileID,';time\t\tdata\trow\tcol\trd\twrt\twl\tcdn\n');
time = sprintf('%012d',str2num(time)+ceil(wait_time/4)); %timestep 0.25 seconds


write_start = time;

row_address = 'A'   ;
col_address = '2' ;
data = 'FF';

fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; setup time \n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; turn on write and wl for selected row \n',time,data,row_address,col_address,'0','1','1','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; hold time \n',time,data,row_address,col_address,'0','0','0','1'); % TODO MAY NEED TO TURN OFF WRITELINE BEFORE turning off others, probably not
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
row_address = '0'   ; %idle
col_address = '0' ;
data = '00';
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; idle \n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period

write_end = time;

%% measure read energy
fprintf(fileID,'\n\n\n; Start measuring read energy\n\n');
fprintf(fileID,';time\t\tdata\trow\tcol\trd\twrt\twl\tcdn\n');
time = sprintf('%012d',str2num(time)+ceil(wait_time/4)); %timestep 0.25 seconds


fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; \n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period

row_address = 'A'   ;
col_address = '2' ;
data = '00';

read_start = time;

fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; set addressess \n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; precharge \n',time,data,row_address,col_address,'1','1','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; disable precharge, WAIT WITH WRITELINES to avoid writing over values\n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ;open writeline\n',time,data,row_address,col_address,'0','0','1','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; turn on read to enable sense amp\n',time,data,row_address,col_address,'1','0','1','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; turn off writeline, data is being latched at this clock cycle\n',time,data,row_address,col_address,'1','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
row_address = '0'   ;
col_address = '0' ;
data = '00';
fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; return to idle\n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period

read_end = time;

fprintf(fileID,'%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s ; idle \n',time,data,row_address,col_address,'0','0','0','1');
time = sprintf('%012d',str2num(time)+period); %timestep one clock period
%% print out stats and close file
fprintf(fileID,'\n\n\n; leak_start = %s \t leak_end = %s \n; write_start = %s \t write_end = %s\n; read_start = %s \t read_end = %s\n',leak_start, leak_end, write_start,write_end,read_start,read_end);
fclose(fileID);
end

